--[[
-----------------------------HOW TO USE THE FUNCTIONS-----------------------
To create your progress bar, use:
element guiCreateColorProgressBar( float x,float  y,float width,float height, bool relative [, parent, image]  )
* x,y - position of the progress bar
* width,height - size of the progress bar
* relative - whether sizes & position are relative 
* parent(OPTIONAL) - the parent the progress bar.  If you dont want a parent(i.e. relative to screen) then pass false
* image(OPTIONAL) - this will override the default barcolor.png and use your own colour for the progress bar.  Images should be in .png format wth a res of 1x1
 RETURNS: The function returns a progress bar gui element

---

To set progress on your progress bar, use:
bool guiColorProgressBarSetProgress ( progressBar bar, float percent, string direction )
* bar - the gui progress bar you want to set progress bar
* percent - a float between 0-100 refering to the percentage you want to set your progress bar
* direction(OPTIONAL) - a string, either "up","down","left","right",  refering to which direction the progress appears.  This is "right" by default.
RETURNS:  Returns a boolean whether the operation was successful

--

To get how much progress your progress bar is currently at, and what direction, use:
float string guiColorProgressBarGetProgress ( progressBar bar )
* bar - The progress bar you want to return progress of
RETURNS: A float of the current progress, and a string of the current direction

--

To set your progress bar to use the colour you added to it, you set the appropriate alpha - use:
bool guiColorProgressBarSetColorAlpha ( progressBar bar, int alpha )
* bar - The progress bar you want to set the colour of
* alpha - The amount of alpha you want it to be.  1 is opaque and 255 is invisible.
RETURNS:  Returns a boolean whether the operation was successful
--

To get your progress bar's colour alpha, use:
int guiColorProgressBarGetColorAlpha ( bar )
* bar - The progress bar you want to return the colour alpha of
RETURNS:  Returns an integer of the current alpha.
]]

local defaultImage = "barcolor.png"
local verticalBars = {}

function guiCreateColorProgressBar( x, y, width, height, relative, parent, image )
	if ( not image ) then image = defaultImage end
	local bar = guiCreateEdit( x, y, width, height, "",relative, parent )
	verticalBars[bar] = {}
	verticalBars[bar]["button"] = guiCreateButton( 0, 0, 0, 0, "",true, bar )
	verticalBars[bar]["red"] = guiCreateStaticImage ( 0, 0, 0, 0, image, true, verticalBars[bar]["button"] )
	guiSetAlpha ( verticalBars[bar]["red"], 0 )
	verticalBars[bar]["progress"] = 0
	return bar
end

function guiColorProgressBarSetProgress ( bar, percent, direction )
	if verticalBars[bar] == nil then return false end
	percent = tonumber(percent)
	if percent == nil then return false end
	verticalBars[bar]["progress"] = percent
	local progressBar = verticalBars[bar]["button"]
	local redBar = verticalBars[bar]["red"]
	percent = percent/100
	local x,y,sizeX,sizeY
	if ( not direction ) then
		direction = "right"
	end
	if direction == "up" then
		x = 0
		y = 1-percent
		sizeX = 1
		sizeY = percent
	elseif direction == "down" then
		x = 0
		y = 0
		sizeX = 1
		sizeY = percent
	elseif direction == "right" then
		x = 0
		y = 0
		sizeX = percent
		sizeY = 1
	elseif direction == "left" then
		x = 1-percent
		y = 0
		sizeX = percent
		sizeY = 1
	else
		return false
	end
	verticalBars[bar]["direction"] = direction
	---resize the button
	local returnValue1 = guiSetSize ( progressBar, sizeX, sizeY, true )
	local returnValue2 = guiSetPosition ( progressBar, x, y, true )
	--resize the image
	local returnValue3 = guiSetSize ( redBar, 1, 1, true )
	local returnValue4 = guiSetPosition ( redBar, 0, 0, true )
	--return any unsuccessful functions
	if returnValue2 == false then return false end
	if returnValue3 == false then return false end
	if returnValue4 == false then return false end
	return returnValue1
end

function guiColorProgressBarGetProgress ( bar )
	return verticalBars[bar]["progress"],verticalBars[bar]["direction"]
end

function guiColorProgressBarSetColorAlpha ( bar, alpha )
	returnValue = guiSetAlpha ( verticalBars[bar]["red"], alpha )
	return returnValue
end

function guiColorProgressBarGetColorAlpha ( bar )
	returnValue = guiGetAlpha ( verticalBars[bar]["red"] )
	return returnValue
end

